<?xml version='1.0'?>
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

	<xsl:variable name="NF_INTEGER" select="'###,##0'" />
	<xsl:variable name="NF_PERCENTAGE" select="'##0%'" />
	<xsl:variable name="SPACE" select="'&#160;'" />

	<xsl:variable name="OPTION_LITE" select="//settings/@lite='true'" />
	<xsl:variable name="OPTION_HIDE_EXTERNALS"
		select="//property[@name='hide-externals']/@value" />

	<xsl:variable name="PROJNAME" select="//model/@local-name" />
	<xsl:variable name="TITLE">
		<xsl:value-of select="$PROJNAME" />
		summary
	</xsl:variable>

	<xsl:variable name="CLASSPATH_AS_APPENDIX"
		select="count(//classpathentry[@kind='lib']) &gt; 3" />

	<xsl:param name="INLINE">
		false
	</xsl:param>
	<xsl:param name="TIPS">
		true
	</xsl:param>
	<xsl:param name="IMAGES">
		none
	</xsl:param>
	<xsl:param name="APP_NAME">
		app-name goes here
	</xsl:param>
	<xsl:param name="APP_URL">
		app-url goes here
	</xsl:param>
	<xsl:param name="PRIMARY_COLOR">
		primary-color goes here
	</xsl:param>

	<xsl:template match="/">
		<html>
			<head>
				<title>
					<xsl:value-of select="$TITLE" />
				</title>
				<xsl:if test="$INLINE='false'">
					<style>

						body {
						border: 1px solid #d4d4d4;
						-webkit-border-radius: 4px;
						-moz-border-radius: 4px;
						border-radius: 4px;
						-webkit-box-shadow: 0 1px 4px rgba(0, 0, 0, 0.065);
						-moz-box-shadow: 0 1px 4px rgba(0, 0, 0, 0.065);
						box-shadow: 0 1px 4px rgba(0, 0, 0, 0.065);
						font-family: sans-serif;
						font-size: 1.0em;
						color: #627883;
						margin: 10;
						}

						body p,body a,body li {
						font-size: 1.0em;
						}

						body a {
						color: #0088cc;
						text-decoration: none;
						cursor: pointer;
						}

						body a:hover {
						text-decoration: underline;
						}

						h2 {
						color: #618454;
						font-weight: bold;
						font-size: 1.3em;
						padding-bottom: 10px;
						border-bottom: 1px solid #d4d4d4;
						clear: left;
						}

						p {
						margin-top: 0;
						margin-bottom: 3;
						}

						li {
						margin-top: 0;
						margin-bottom: 0;
						}

						table.inbrowser {
						border: 1px solid #d4d4d4;
						}

						table.inbrowser th {
						color: #FFFFFF;
						text-align: left;
						font-size: 0.8em;
						font-weight bold;
						padding: 0px 5px 0px 5px;
						}

						table {
						cellpadding: 0;
						}

						table thead {
						background: #8fa7a7;
						color: #FFFFFF;
						padding: 0px 0px 0px 5px;
						}

						table th {
						color: #FFFFFF;
						text-align: left;
						font-size: 1.0em;
						font-weight bold;
						padding: 0px 0px 0px 0px;
						}

						table td {
						font-size: 0.9em;
						padding: 0px 5px 0px 5px;
						}

						div#container {
						margin: 10;
						}

						div#navbar {
						height: 50px;
						padding-left: 20px;
						padding-right: 20px;
						background-color: #fafafa;
						background-image: -moz-linear-gradient(top, #ffffff, #f2f2f2);
						background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#ffffff),
						to(#f2f2f2));
						background-image: -webkit-linear-gradient(top, #ffffff, #f2f2f2);
						background-image: -o-linear-gradient(top, #ffffff, #f2f2f2);
						background-image: linear-gradient(to bottom, #ffffff, #f2f2f2);
						background-repeat: repeat-x;
						border-bottom: 1px solid #d4d4d4;
						-webkit-border-radius: 4px;
						-moz-border-radius: 4px;
						border-radius: 4px;
						-webkit-box-shadow: 0 1px 4px rgba(0, 0, 0, 0.065);
						-moz-box-shadow: 0 1px 4px rgba(0, 0, 0, 0.065);
						box-shadow: 0 1px 4px rgba(0, 0, 0, 0.065);
						color: #618454; !important;
						text-shadow: 0 1px 0 rgba(255, 255, 255, 0.1), 0 0 30px rgba(255, 255, 255,
						0.125);
						font-weight: bold !important;
						}

						div#copyright {
						margin-top:10px;
						height: 35px;
						background-color: #fafafa;
						background-image: -moz-linear-gradient(top, #ffffff, #f2f2f2);
						background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#ffffff),
						to(#f2f2f2));
						background-image: -webkit-linear-gradient(top, #ffffff, #f2f2f2);
						background-image: -o-linear-gradient(top, #ffffff, #f2f2f2);
						background-image: linear-gradient(to bottom, #ffffff, #f2f2f2);
						background-repeat: repeat-x;
						border-top: 1px solid #d4d4d4;
						-webkit-border-radius: 4px;
						-moz-border-radius: 4px;
						border-radius: 4px;
						-webkit-box-shadow: 0 1px 4px rgba(0, 0, 0, 0.065);
						-moz-box-shadow: 0 1px 4px rgba(0, 0, 0, 0.065);
						box-shadow: 0 1px 4px rgba(0, 0, 0, 0.065);
						}

						div#copyright p {
						margin: 15px 0 0 10px;
						font-size: 0.8em;
						}
					</style>
				</xsl:if>
			</head>
			<body vLink="#ff9900" aLink="#E65A1D" link="#E65A1D">
				<div id="navbar">
					<h1>
						<xsl:value-of select="$TITLE" />
					</h1>
				</div>
				<div id="container">
					<table cellpadding="1">
						<xsl:apply-templates select="//model" />
						<xsl:if test="//hiview/@filtered='true'">
							<xsl:call-template name="summary-row">
								<xsl:with-param name="caption">
									Hiding
								</xsl:with-param>
								<xsl:with-param name="block">
									On - some items and/or dependencies have been interactively
									hidden from the view
								</xsl:with-param>
							</xsl:call-template>
						</xsl:if>
						<xsl:call-template name="summary-row">
							<xsl:with-param name="caption">
								Hierarchy
							</xsl:with-param>
							<xsl:with-param name="block">
								"
								<xsl:value-of select="//hiview/@hierarchy" />
								"
								<xsl:if test="//hiview/@hierarchy!='Package'">
									- XS measurements not available for this hierarchy
								</xsl:if>
							</xsl:with-param>
						</xsl:call-template>
						<xsl:call-template name="summary-row">
							<xsl:with-param name="caption">
								Notes and tips
							</xsl:with-param>
							<xsl:with-param name="block">
								<xsl:choose>
									<xsl:when test="$TIPS='true'">
										On
									</xsl:when>
									<xsl:otherwise>
										Off
									</xsl:otherwise>
								</xsl:choose>
							</xsl:with-param>
						</xsl:call-template>
						<xsl:call-template name="summary-row">
							<xsl:with-param name="caption">
								Created
							</xsl:with-param>
							<xsl:with-param name="block">
								<xsl:value-of select="/report/@date" />
								<xsl:value-of select="$SPACE" />
								<xsl:value-of select="/report/@time" />
								by
								<a href="fx_{$APP_URL}">
									<xsl:value-of select="$APP_NAME" />
								</a>
							</xsl:with-param>
						</xsl:call-template>
					</table>

					<xsl:variable name="externals-tip">
						<xsl:if test="$OPTION_HIDE_EXTERNALS='false'">
							<li>External classes are flagged in the UI with a "!" adornment
								to the icon.</li>
						</xsl:if>
					</xsl:variable>

					<xsl:call-template name="tips">
						<xsl:with-param name="tips">
							<li>This summary has been generated with "Show notes and tips"
								enabled - these are shown
								with a yellow background, like this one.
								You can switch them off via the drop-down menu in the Overview
								perspective (or with the
								-notips switch from the command line).</li>
							<xsl:copy-of select="$externals-tip" />
							<li>The Collaboration perspective is great for tracking down your
								dependency on third party libraries.
								Set "Show externals" in the project properties and then select the
								library item in the right-hand
								tree - anything that doesn't use it will be grayed out in the
								left-hand tree.</li>

						</xsl:with-param>
					</xsl:call-template>

					<xsl:apply-templates select="//size" />
					<xsl:apply-templates select="//violations" />
					<xsl:apply-templates select="//codemap_stats" />
					<xsl:apply-templates select="//parse-errors" />
					<xsl:apply-templates select="//tanglicities" />

					<h2>Excessive Structural Complexity (XS)</h2>
					<xsl:choose>
						<xsl:when test="//xs-config">
							<xsl:apply-templates select="//xs-config" />
						</xsl:when>
						<xsl:otherwise>
							<p>
								This section of the report has been excluded. To include, please
								deselect
								"Exclude XS metrics" in the Options menu.
							</p>
						</xsl:otherwise>
					</xsl:choose>

					<xsl:if test="$CLASSPATH_AS_APPENDIX='true'">
						<a name="classpath" />
						<h2>Appendix 1 - Project Classpath</h2>
						<p>
							<xsl:for-each select="//classpathentry[@kind='lib']">
								<xsl:value-of select="@path" />
								<br />
							</xsl:for-each>
						</p>
					</xsl:if>

				</div> <!-- container -->
				<div id="copyright">
					<p>
						<small>
							Created by
							<a href="fx_{$APP_URL}">
								<xsl:value-of select="$APP_NAME" />
								for Java
							</a>
							(Build no.
							<xsl:value-of select="report/@version" />
							)
						</small>
					</p>
				</div>
			</body>
		</html>
	</xsl:template>

	<xsl:template match="model">
		<xsl:if test="@location">
			<xsl:call-template name="summary-row">
				<xsl:with-param name="caption">
					Project file
				</xsl:with-param>
				<xsl:with-param name="block">
					<xsl:value-of select="@location" />
				</xsl:with-param>
			</xsl:call-template>
		</xsl:if>
		<xsl:apply-templates select="settings" />
	</xsl:template>

	<xsl:template match="local-project">
		<xsl:call-template name="summary-row">
			<xsl:with-param name="caption">
				Classpath
			</xsl:with-param>
			<xsl:with-param name="block">
				<xsl:choose>
					<xsl:when test="$CLASSPATH_AS_APPENDIX='true' and $INLINE='true'">
						See Appendix 1
					</xsl:when>
					<xsl:when test="$CLASSPATH_AS_APPENDIX='true'">
						See
						<a href="#classpath">Appendix 1</a>
					</xsl:when>
					<xsl:otherwise>
						<xsl:for-each select="//classpathentry[@kind='lib']">
							<xsl:value-of select="@path" />
							<br />
						</xsl:for-each>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="summary-row">
			<xsl:with-param name="caption">
				Granularity
			</xsl:with-param>
			<xsl:with-param name="block">
				<xsl:choose>
					<xsl:when test="not($OPTION_LITE)">
						"Detail" - includes inner classes, methods and fields
					</xsl:when>
					<xsl:otherwise>
						"Overview" - no inner classes, methods or fields (fat classes and
						methods not detected)
					</xsl:otherwise>
				</xsl:choose>
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="summary-row">
			<xsl:with-param name="caption">
				Externals
			</xsl:with-param>
			<xsl:with-param name="block">
				<xsl:choose>
					<xsl:when test="$OPTION_HIDE_EXTERNALS='true'">
						"Hide" - referenced items are not included
					</xsl:when>
					<xsl:otherwise>
						"Show" - includes items that are referenced by the project
						(distorts some statistics and XS measurements)
					</xsl:otherwise>
				</xsl:choose>
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="summary-row">
			<xsl:with-param name="caption">
				Excludes
			</xsl:with-param>
			<xsl:with-param name="block">
				<xsl:choose>
					<xsl:when test="excludes/exclude">
						"Yes" - User-defined items have been excluded from the model
					</xsl:when>
					<xsl:otherwise>
						None
					</xsl:otherwise>
				</xsl:choose>
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="summary-row">
			<xsl:with-param name="caption">
				Transformations
			</xsl:with-param>
			<xsl:with-param name="block">
				<xsl:choose>
					<xsl:when test="transformations/transformation">
						"Yes" - User-defined transformations have been applied to the
						package hierarchy
					</xsl:when>
					<xsl:otherwise>
						None
					</xsl:otherwise>
				</xsl:choose>
			</xsl:with-param>
		</xsl:call-template>
	</xsl:template>


	<xsl:template name="summary-row">
		<xsl:param name="caption" />
		<xsl:param name="block" />
		<tr>
			<td valign="top" nowrap="nowrap">
				<xsl:copy-of select="$caption" />
				:
				<xsl:value-of select="$SPACE" />
				<xsl:value-of select="$SPACE" />
				<xsl:value-of select="$SPACE" />
			</td>
			<td>
				<xsl:copy-of select="$block" />
			</td>
		</tr>
	</xsl:template>

	<xsl:template match="size">
		<h2>Size</h2>
		<table>
			<tr>
				<td valign="top">
					<table cellpadding="1">
						<tr>
							<td>
								Jars (and/or classpath directories):
								<xsl:value-of select="$SPACE" />
								<xsl:value-of select="$SPACE" />
								<xsl:value-of select="$SPACE" />
							</td>
							<td align="right">
								<xsl:value-of select="format-number(@jars, $NF_INTEGER)" />
							</td>
						</tr>
						<tr>
							<td>Packages (that contain classes):</td>
							<td align="right">
								<xsl:value-of select="format-number(@packages, $NF_INTEGER)" />
							</td>
						</tr>
						<tr>
							<td>Classes (outer):</td>
							<td align="right">
								<xsl:variable name="OUTER_CLASS">
									<xsl:value-of select="format-number(@classes-outer, $NF_INTEGER)" />
								</xsl:variable>
								<xsl:value-of select="format-number(@classes-outer, $NF_INTEGER)" />
							</td>
						</tr>
						<tr>
							<td>Classes (all):</td>
							<td align="right">
								<xsl:value-of select="format-number(@classes-all, $NF_INTEGER)" />
							</td>
						</tr>
						<tr>
							<td>Classes (external):</td>
							<td align="right">
								<xsl:value-of select="format-number(//externals/@count, $NF_INTEGER)" />
							</td>
						</tr>
					</table>
				</td>
				<td width="30">
					<xsl:value-of select="$SPACE" />
				</td>
				<td valign="top">
					<table cellpadding="1">
						<tr>
							<td>NI (Number of bytecode Instructions): </td>
							<td align="right">
								<xsl:choose>
									<xsl:when test="@ni &gt; 10000">
										<xsl:value-of select="format-number(@ni div 1000, $NF_INTEGER)" />
										K
									</xsl:when>
									<xsl:otherwise>
										<xsl:value-of select="format-number(@ni, $NF_INTEGER)" />
									</xsl:otherwise>
								</xsl:choose>
							</td>
						</tr>
						<tr>
							<td>
								LOC (Non Comment Non Blank Lines Of Code):
								<xsl:value-of select="$SPACE" />
								<xsl:value-of select="$SPACE" />
								<xsl:value-of select="$SPACE" />
							</td>
							<td align="right">
								<xsl:choose>
									<xsl:when test="@ni &gt; 10000">
										~
										<xsl:value-of
											select="format-number(@ni * 0.43 div 1000, $NF_INTEGER)" />
										K
									</xsl:when>
									<xsl:otherwise>
										~
										<xsl:value-of select="format-number(@ni * 0.43, $NF_INTEGER)" />
									</xsl:otherwise>
								</xsl:choose>
							</td>
						</tr>
					</table>
				</td>
			</tr>
		</table>
		<xsl:call-template name="tips">
			<xsl:with-param name="notes">
				<li>Though less commonly used than LOC, NI is an excellent,
					style-independent, relative measure of code size.</li>
				<li>The value for LOC is estimated from the NI.</li>
			</xsl:with-param>
		</xsl:call-template>
	</xsl:template>
	<xsl:template match="codemap_stats">
		<h2>Model</h2>
		<table>
			<tr>
				<td>Model:</td>
				<td>
					<xsl:value-of select="@current_model" />
				</td>
			</tr>
			<xsl:variable name="CURRENT_MODEL">
				<xsl:value-of select="@current_model" />
			</xsl:variable>
			<xsl:if test="$CURRENT_MODEL!='none'">
				<tr>
					<td>
						#sequential actions:
					</td>
					<td align="right">
						<xsl:value-of select="format-number(@sequential_actions,$NF_INTEGER)" />
					</td>
				</tr>
				<tr>
					<td>
						#refactoring actions:
					</td>
					<td align="right">
						<xsl:value-of select="format-number(@refactoring_actions,$NF_INTEGER)" />
					</td>
				</tr>
				<tr>
					<td>#class mappings:</td>

					<td align="right">
						<xsl:value-of select="format-number(@class_mappings,$NF_INTEGER)" />
					</td>
				</tr>
				<tr>
					<td>Alignment:</td>
					<td align="right">
						<xsl:value-of select="format-number(@alignment,$NF_PERCENTAGE)" />
					</td>
				</tr>
			</xsl:if>
		</table>
		<xsl:call-template name="tips">
			<xsl:with-param name="notes">
				<li>The model organizes the classes in your codebase into a
					compositional hierarchy</li>
				<li>You perform actions interactively to organize your classes into
					the model hierarchy, and to define refactoring actions. The model
					shows you the effect of these actions.</li>
				<li>“#sequential actions” is a count of the number of separate
					actions you have performed – this is indicative of the amount of
					modelling work you have done so far.</li>
				<li>“#refactoring actions” is the number of implementation-level
					changes you have made to the model. These are actions that do not
					relocate classes between containers, but that imply changes to the
					code contained by classes.</li>
				<li>“#class mappings” is a count of the number of classes that are
					located in a location that does not correspond to a physical
					container in the code-base.</li>
				<li>You may want the physical code base structure (e.g. packages,
					jars, Maven projects) to reflect the model structure. The degree to
					which the physical structure matches the model is called
					“alignment”
					and is given as a percentage above.</li>
				<li>Applying all the class mappings to the code-base would result in
					alignment = 100%.</li>
			</xsl:with-param>
			<xsl:with-param name="tips">
				<li>You can initialize the model using physical structures (such as
					packages or jars) from your code-base, or get Structure101 to
					suggest a start model by finding cohesive clusters of classes
					(“auto-levelize” from the context menu).</li>
				<li>To align the physical code with the model, select the “class
					map”
					option in the publish wizard. The mappings will appear as actions in
					the IDE plugin.</li>
				<li>If you do not wish to undertake alignment (for now), but have
					simulated implementation-level changes in the model, you can export
					these by selecting “refactorings” in the publish wizard. The list
					of
					refactoring actions will appear as actions in the IDE plugin.</li>
				<li>If you want to perform both mapping and refactoring actions in
					the same sequence as you performed them on the model, select
					“Sequential action list” in the publish wizard.</li>
			</xsl:with-param>
		</xsl:call-template>
	</xsl:template>
	<xsl:template match="parse-errors[@count &gt; 0]">
		<h2>Parse errors</h2>
		<p>
			Parse errors were encountered on the following class files during the
			reverse engineering process:
			<ul>
				<xsl:for-each select="hinode">
					<li>
						<xsl:value-of select="@name" />
					</li>
				</xsl:for-each>
				<xsl:if test="@truncated-at">
					<li>
						etc. (total of
						<xsl:value-of select="format-number(@count, $NF_INTEGER)" />
						)
					</li>
				</xsl:if>
			</ul>
		</p>
		<xsl:call-template name="tips">
			<xsl:with-param name="notes">
				<li>
					Classes where the parse failed are flagged in the UI with a red
					"stop-sign" adornment to the icon.
				</li>
				<li>
					We are proud of the accuracy and reliability of our Java class parser
					- at time of writing, we are
					not aware of any problems dealing with valid class files. If you think
					this is a bug, please email
					us support, attaching one of the problem class files.
				</li>
			</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="violations">
		<h2>Architecture</h2>
		<xsl:choose>
			<xsl:when test="diagram">
				<table cellpadding="2">
					<tr>
						<td>#Diagrams: </td>
						<td align="right">
							<xsl:value-of select="count(diagram)" />
						</td>
					</tr>
					<tr>
						<td>#Violations: </td>
						<td align="right">
							<xsl:value-of select="@total-weighted" />
						</td>
					</tr>
					<tr>
						<xsl:variable name="violationfrequency"
							select="@total-weighted div @population-weighted" />
						<td>Violation frequency: </td>
						<td align="right">
							<xsl:choose>
								<xsl:when
									test="$violationfrequency > 0.00 and not($violationfrequency >= .01)">
									<xsl:value-of select="format-number(.01, $NF_PERCENTAGE)" />
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of
										select="format-number($violationfrequency, $NF_PERCENTAGE)" />
								</xsl:otherwise>
							</xsl:choose>
						</td>
						<!--td>[<xsl:value-of select="$violationfrequency"/>][<xsl:value-of 
							select="@total-weighted"/> / <xsl:value-of select="@population-weighted"/>]</td -->
					</tr>
					<xsl:apply-templates select="//unassociated-items" />
				</table>
			</xsl:when>
			<xsl:otherwise>
				<p>The architecture for this project is not defined</p>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:call-template name="tips">
			<xsl:with-param name="notes">
				<li>
					The architecture is defined in the form of a number of architecture
					diagrams that define layering and
					composition, and map to the currently selected model.
				</li>
				<li>
					Architecture diagrams are not automatically created - that must be done by
					users (e.g. you).
				</li>
				<li>
					The "#Violations" value is the number of class-to-class dependencies
					if the project
					granularity is "Overview" (see the top of this report). If the granularity is
					"Detail" then it is
					the number of method/field/class dependencies.
				</li>
				<li>
					The "Violation frequency" is the number of violations as a percentage
					of all the dependencies
					in the model.
				</li>
			</xsl:with-param>
			<xsl:with-param name="tips">
				<li>
					You can initialize new diagrams wherever you see the "new
					architectural diagram"
					button - in the Composition perspective, the Slice perspective and the
					rules tab.
				</li>
				<li>
					Create architecture diagrams from dependency diagrams that most closely
					reflect the intended architecture.
				</li>
				<li>
					You can also create architecture diagrams from scratch in the rules
					tab.
				</li>
				<li>
					Once you have created architectural diagrams, you can edit them in the
					rules tab.
				</li>
				<li>
					If you publish Architecture diagrams to a repository, the
					<xsl:value-of select="$APP_NAME" />
					IDE plug-ins will warn
					developers if they make code changes that are inconsistent with them, plus
					they will be browsable via the
					<xsl:value-of select="$APP_NAME" />
					web application.
				</li>
			</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="unassociated-items">
		<tr>
			<td>#Unassociated items: </td>
			<td align="right">
				<xsl:value-of select="@unique-items" />
			</td>
		</tr>
	</xsl:template>

	<xsl:template match="tanglicities">
		<h2>Flat Tangles</h2>
		<xsl:call-template name="my-table">
			<xsl:with-param name="content">
				<thead>
					<xsl:call-template name="my-th">
						<xsl:with-param name="content">
							Level
						</xsl:with-param>
						<xsl:with-param name="align">
							left
						</xsl:with-param>
					</xsl:call-template>
					<xsl:call-template name="my-th">
						<xsl:with-param name="content">
							#Items
						</xsl:with-param>
					</xsl:call-template>
					<xsl:call-template name="my-th">
						<xsl:with-param name="content">
							#Tangles
						</xsl:with-param>
					</xsl:call-template>
					<xsl:call-template name="my-th">
						<xsl:with-param name="content">
							#Tangled items
						</xsl:with-param>
					</xsl:call-template>
					<xsl:call-template name="my-th">
						<xsl:with-param name="content">
							Biggest
						</xsl:with-param>
					</xsl:call-template>
					<xsl:call-template name="my-th">
						<xsl:with-param name="content">
							Degree
						</xsl:with-param>
					</xsl:call-template>
				</thead>
				<xsl:apply-templates />
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="tips">
			<xsl:with-param name="notes">
				<li>"Flat" tangles are detected and measured from the flat
					dependency graphs of all
					items at a specific level. These are independent of the "Design" levels
					which are
					created by the higher-level packaging and are measured and reported in the
					"Excessive Structural Complexity (XS)" section below.</li>
				<li>Tangles at the Jar or Package-level are highly disruptive.</li>
				<li>The bigger the tangle, the more problems it is likely to cause.</li>
				<li>Larger tangles at the class level are likely to cross package
					boundaries and so may also
					indicate a problem.</li>
				<li>The degree is calculated so as to give a higher value for larger
					tangles - that is,
					one large tangle will give a higher degree than the same number of
					items involved in
					several smaller tangles.</li>
			</xsl:with-param>
			<xsl:with-param name="tips">
				<li>The "Leaf package" and "Outer class" tangles can be directly
					viewed and analyzed in the Slice perspective.</li>
				<li>To view and analyze jar-level tangles, select the Jar breakout
					in Project Properties.</li>
			</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="tanglicity">
		<tr>
			<td>
				<xsl:value-of select="@level" />
			</td>
			<td align="right">
				<xsl:value-of select="format-number(@nodes, $NF_INTEGER)" />
			</td>
			<xsl:choose>
				<xsl:when test="@nodes &gt; 1">
					<td align="right">
						<xsl:value-of select="format-number(@tangles, $NF_INTEGER)" />
					</td>
					<td align="right">
						<xsl:value-of select="format-number(@tangled-nodes, $NF_INTEGER)" />
					</td>
					<td align="right">
						<xsl:value-of select="format-number(@biggest, $NF_INTEGER)" />
					</td>
					<td align="right">
						<xsl:value-of select="format-number(@tanglicity, $NF_PERCENTAGE)" />
					</td>
				</xsl:when>
				<xsl:otherwise>
					<td align="right">n/a</td>
					<td align="right">n/a</td>
					<td align="right">n/a</td>
					<td align="right">n/a</td>
				</xsl:otherwise>
			</xsl:choose>
		</tr>
	</xsl:template>

	<xsl:template match="xs-config">
		<table border="0" cellpadding="1">
			<tr>
				<td>Cumulative XS:</td>
				<td>
					<xsl:choose>
						<xsl:when test="xs-summary/summary/@cumulative-xs">
							<xsl:value-of
								select="format-number(xs-summary/summary/@cumulative-xs, $NF_INTEGER)" />
						</xsl:when>
						<xsl:otherwise>
							<i>Not available for this hierarchy</i>
						</xsl:otherwise>
					</xsl:choose>
				</td>
			</tr>
			<tr>
				<td>Average XS:</td>
				<td>
					<xsl:choose>
						<xsl:when test="xs-summary/summary/@average-xs">
							<xsl:value-of
								select="format-number(xs-summary/summary/@average-xs, $NF_PERCENTAGE)" />
						</xsl:when>
						<xsl:otherwise>
							<i>Not available for this hierarchy</i>
						</xsl:otherwise>
					</xsl:choose>
				</td>
			</tr>
		</table>

		<xsl:call-template name="tips">
			<xsl:with-param name="notes">
				<li>The XS Configuration is a set of thresholds for "Fat" and
					"Tangles" at different levels of composition.</li>
				<li>Fat indicates "too much stuff" in one place and is measurable at
					every level of composition - Design,
					Package, Class and Method.</li>
				<li>Cyclomatic Complexity is used as a measure of Fat at the method
					level - the number of dependencies
					in the child dependency graph is used at the other levels.</li>
				<li>For the purposes of XS, tangles are only measured at the design
					level, i.e. cyclic dependencies between packages.</li>
				<li>Both Fat and Tangles are measured as the percentage by which the
					threshold is exceeded. This percentage is
					multiplied by the size of the item so that the Average XS can be read as a
					percentage of the code-base.</li>
				<li>
					Read
					<a href="fx_http://www.structure101.com/xs">more</a>
					about the concepts behind XS.
				</li>
			</xsl:with-param>
			<xsl:with-param name="tips">
				<li>Use the Measurement perspective to obtain detailed information
					on the XS in your project.</li>
				<li>Change the metrics and thresholds used to calculate XS by
					putting the XS Configuration
					into "Edit mode" from the drop down menu, bottom-left, in the
					Measurement perspective.
					Save the changes as a new named configuration.</li>
				<li>If you have a a very large project and find the metrics
					calculations too time-consuming, you
					can exclude this section of the report via the Options menu.</li>
			</xsl:with-param>
		</xsl:call-template>

		<xsl:apply-templates select="xs-summary/summary" />
		<xsl:apply-templates select="xs-summary/offenders" />

	</xsl:template>

	<xsl:template match="summary">
		<h3>XS breakout by metric (and scope)</h3>
		<xsl:call-template name="xs-slices" />
		<xsl:apply-templates select="slice" />
	</xsl:template>

	<xsl:template name="xs-slices">
		<xsl:call-template name="my-table">
			<xsl:with-param name="content">
				<thead>
					<xsl:call-template name="my-th">
						<xsl:with-param name="content">
							Metric (and scope)
						</xsl:with-param>
						<xsl:with-param name="align">
							left
						</xsl:with-param>
					</xsl:call-template>
					<xsl:call-template name="my-th">
						<xsl:with-param name="content">
							Threshold
						</xsl:with-param>
					</xsl:call-template>
					<xsl:call-template name="my-th">
						<xsl:with-param name="content">
							#Offenders
						</xsl:with-param>
					</xsl:call-template>
					<xsl:call-template name="my-th">
						<xsl:with-param name="content">
							Offenses (%)
						</xsl:with-param>
					</xsl:call-template>
					<xsl:call-template name="my-th">
						<xsl:with-param name="content">
							XS contribution
						</xsl:with-param>
					</xsl:call-template>
				</thead>
				<xsl:for-each select="slice">
					<tr>
						<td>
							<xsl:value-of select="@metric" />
							(
							<xsl:value-of select="@scope" />
							)
						</td>
						<td align="right">
							<xsl:variable name="metric" select="@metric" />
							<xsl:variable name="scope" select="@scope" />
							<xsl:value-of
								select="//entry[@metric=$metric and @scope=$scope]/@threshold" />
						</td>
						<td align="right">
							<xsl:value-of select="format-number(@num-offenders, $NF_INTEGER)" />
							of
							<xsl:value-of select="format-number(@num-contributors, $NF_INTEGER)" />
						</td>
						<td align="right">
							<xsl:choose>
								<xsl:when test="@num-contributors &gt; 0">
									<xsl:value-of
										select="format-number(@num-offenders div @num-contributors, $NF_PERCENTAGE)" />
								</xsl:when>
								<xsl:otherwise>
									0%
								</xsl:otherwise>
							</xsl:choose>
						</td>
						<td align="right">
							<xsl:value-of select="format-number(@proportion-of-xs, $NF_PERCENTAGE)" />
						</td>
					</tr>
				</xsl:for-each>
				<tr>
					<td>
						<b>Total</b>
					</td>
					<td colspan="3">
						<xsl:value-of select="$SPACE" />
					</td>
					<td align="right">
						<b>100%</b>
					</td>
				</tr>
			</xsl:with-param>
		</xsl:call-template>

		<xsl:call-template name="tips">
			<xsl:with-param name="notes">
				<li>The Metric (and scope) column contains all the metrics that are
					enabled in the selected XS Configuration
					(named in the previous section).</li>
				<li>The "Threshold" is taken from the selected XS Configuration</li>
				<li>The "#Offenders" indicates the number of items that exceed the
					threshold.</li>
				<li>The "Offenders (%)" indicates the number of items that exceed
					the threshold as a percentage of
					the total number of items at that level.</li>
				<li>The "XS contribution" indicates the degree to which design
					tangles and fat at each level contribute to
					the total amount of XS in the project.</li>
			</xsl:with-param>
			<xsl:with-param name="tips">
				<li>Use the Measurement perspective to obtain detailed information
					on the XS in your project.</li>
			</xsl:with-param>
		</xsl:call-template>

	</xsl:template>

	<xsl:template match="slice">
		<xsl:choose>
			<xsl:when test="@num-offenders &gt; 0">
				<h3>
					<xsl:value-of select="@metric" />
					(
					<xsl:value-of select="@scope" />
					):
					<xsl:choose>
						<xsl:when test="offenders/@truncated-at">
							Top
							<xsl:value-of select="offenders/@truncated-at" />
							(of
							<xsl:value-of select="@num-offenders" />
							)
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="@num-offenders" />
						</xsl:otherwise>
					</xsl:choose>
				</h3>
				<xsl:call-template name="my-table">
					<xsl:with-param name="content">
						<thead>
							<xsl:call-template name="my-th">
								<xsl:with-param name="content">
									Item
								</xsl:with-param>
								<xsl:with-param name="align">
									left
								</xsl:with-param>
							</xsl:call-template>
							<xsl:call-template name="my-th">
								<xsl:with-param name="content">
									Value
								</xsl:with-param>
							</xsl:call-template>
						</thead>
						<xsl:for-each select="offenders/hinode">
							<tr>
								<td>
									<xsl:if test="not($IMAGES='none')">
										<img src="{$IMAGES}/{@type}.gif" align="absmiddle" hspace="1"
											width="16" height="16" />
									</xsl:if>
									<xsl:value-of select="@name" />
								</td>
								<td align="right">
									<xsl:choose>
										<xsl:when test="../../@metric='Tangled'">
											<xsl:value-of select="format-number(@value, $NF_PERCENTAGE)" />
										</xsl:when>
										<xsl:otherwise>
											<xsl:value-of select="format-number(@value, $NF_INTEGER)" />
										</xsl:otherwise>
									</xsl:choose>
								</td>
							</tr>
						</xsl:for-each>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<h3>
					<xsl:value-of select="@metric" />
					(
					<xsl:value-of select="@scope" />
					)
				</h3>
				<p>
					No items exceed the threshold for
					<xsl:value-of select="@metric" />
					at the
					<xsl:value-of select="@scope" />
					level.
				</p>
			</xsl:otherwise>
		</xsl:choose>

		<xsl:call-template name="tips">
			<xsl:with-param name="notes">
				<xsl:choose>
					<xsl:when test="@metric='Tangled' and @scope='design'">
						<li>The "design" level items are high-level packages (i.e.
							packages that contain sub-packages).
							Tangles are only measured at this level.</li>
						<li>The value is calculated as the number of dependencies that go
							bottom-up as a percentage of
							all the dependencies on the child graph for the item.</li>
					</xsl:when>
					<xsl:when test="@metric='Fat' and @scope='design'">
						<li>The "design" level items are high-level packages (i.e.
							packages that contain sub-packages).</li>
						<li>The value is calculated as the number of dependencies in the
							child graph for the item, i.e.
							the dependency graph of the sub-packages of the measured package.</li>
					</xsl:when>
					<xsl:when test="@metric='Fat' and @scope='leaf package'">
						<li>The "leaf package" level items are packages that contain
							classes.</li>
						<li>The value is calculated as the number of dependencies in the
							child graph for the item,
							i.e. the dependency graph of the classes in the measured package</li>
					</xsl:when>
					<xsl:when test="@metric='Fat' and @scope='class'">
						<li>The value is calculated as the number of dependencies in the
							child graph for the item,
							i.e. the dependency graph of the methods and fields in the class.</li>
						<li>This is not available if the Granularity for the project is
							"Overview".</li>
					</xsl:when>
					<xsl:when test="@metric='Fat' and @scope='method'">
						<li>The value is calculated as the Cyclomatic Complexity (McCabe's
							metric) of the method.</li>
						<li>The value is calculated from the bytecode and so may differ
							from the value that would
							be calculated from the corresponding source code in some (rare)
							cases.</li>
						<li>This is not available if the Granularity for the project is
							"Overview".</li>
					</xsl:when>
				</xsl:choose>
			</xsl:with-param>
			<xsl:with-param name="tips">
				<xsl:choose>
					<xsl:when test="@metric='Tangled' and @scope='design'">
						<li>To view and analyze these tangles, drill down to the item in
							the Composition perspective
							with the Package hierarchy selected from the Model menu.</li>
					</xsl:when>
					<xsl:when test="@metric='Fat' and @scope='design'">
						<li>To view and analyze these Fat items, drill down to the item in
							the Composition perspective with
							the Package hierarchy selected from the Model menu.</li>
						<li>Use the "Auto-partition" button on the Dependency Graph to
							help make sense of big graphs.</li>
					</xsl:when>
					<xsl:when test="@metric='Fat' and @scope='leaf package'">
						<li>To view and analyze these Fat items, drill down to the item in
							the Composition perspective.</li>
						<li>Use the "Auto-partition" button on the Dependency Graph to
							help make sense of big graphs.</li>
					</xsl:when>
					<xsl:when test="@metric='Fat' and @scope='class'">
						<li>To view and analyze these Fat items, drill down to the item in
							the Composition perspective.</li>
						<li>Use the "Auto-partition" button on the Dependency Graph to
							help make sense of big graphs.</li>
					</xsl:when>
					<xsl:when test="@metric='Fat' and @scope='method'">
						<li>To view and analyze these Fat items, drill down to the class
							that contains the method
							in any perspective.</li>
						<li>Click on the "Show source" button on the main toolbar to view
							the source.</li>
					</xsl:when>
				</xsl:choose>
				<li>
					The complete list is available in the Measurement perspective. Make
					sure you are at the root of the
					"Drill-down" in the top-left, and then select the
					<xsl:value-of select="@metric" />
					(
					<xsl:value-of select="@scope" />
					)
					slice of the pie. The "Items with XS"
					will contain all the items that exceed the threshold at this level.
				</li>
			</xsl:with-param>
		</xsl:call-template>

	</xsl:template>


	<xsl:template match="offenders">
		<xsl:choose>
			<xsl:when test="@count &gt; 0 and @truncated-at">
				<h3>
					Items with highest XS - Top
					<xsl:value-of select="@truncated-at" />
					(of
					<xsl:value-of select="format-number(@count, $NF_INTEGER)" />
					)
				</h3>
			</xsl:when>
			<xsl:when test="@count &gt; 0">
				<h3>
					Items with highest XS:
					<xsl:value-of select="@count" />
				</h3>
			</xsl:when>
			<xsl:otherwise>
				<h3>Items with highest XS</h3>
				<p>None</p>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:if test="@count &gt; 0">
			<xsl:call-template name="my-table">
				<xsl:with-param name="content">
					<thead>
						<xsl:call-template name="my-th">
							<xsl:with-param name="content">
								Item
							</xsl:with-param>
							<xsl:with-param name="align">
								left
							</xsl:with-param>
						</xsl:call-template>
						<xsl:if test="$IMAGES='none'">
							<xsl:call-template name="my-th">
								<xsl:with-param name="content">
									Type
								</xsl:with-param>
								<xsl:with-param name="align">
									left
								</xsl:with-param>
							</xsl:call-template>
						</xsl:if>
						<xsl:call-template name="my-th">
							<xsl:with-param name="content">
								Tangled
							</xsl:with-param>
						</xsl:call-template>
						<xsl:call-template name="my-th">
							<xsl:with-param name="content">
								Fat
							</xsl:with-param>
						</xsl:call-template>
						<xsl:call-template name="my-th">
							<xsl:with-param name="content">
								Size
							</xsl:with-param>
						</xsl:call-template>
						<xsl:call-template name="my-th">
							<xsl:with-param name="content">
								XS
							</xsl:with-param>
						</xsl:call-template>
					</thead>
					<xsl:for-each select="hinode">
						<tr>
							<td style="padding-right: 5">
								<xsl:if test="not($IMAGES='none')">
									<img src="{$IMAGES}/{@type}.gif" align="absmiddle" hspace="1"
										width="16" height="16" />
								</xsl:if>
								<xsl:value-of select="@name" />
							</td>
							<xsl:if test="$IMAGES='none'">
								<td>
									<xsl:value-of select="@type" />
								</td>
							</xsl:if>
							<td style="padding-left: 5" align="right">
								<xsl:value-of select="$SPACE" />
								<xsl:apply-templates select="metric[@name='Tangled']" />
							</td>
							<td align="right">
								<xsl:value-of select="$SPACE" />
								<xsl:apply-templates select="metric[@name='Fat']" />
							</td>
							<td style="padding-left: 5" align="right">
								<xsl:value-of select="format-number(@size, $NF_INTEGER)" />
							</td>
							<td style="padding-left: 5" align="right">
								<xsl:value-of select="format-number(@xs,  $NF_INTEGER)" />
							</td>
						</tr>
					</xsl:for-each>
				</xsl:with-param>
			</xsl:call-template>
		</xsl:if>

		<xsl:call-template name="tips">
			<xsl:with-param name="notes">
				<li>XS is related to the value and the size of the item so that
					bigger items (usually higher in the hierarchy)
					score higher. Since excessive structural complexity on bigger items
					(e.g. packages) will tend to disrupt
					development activities more than on smaller items (e.g. methods), you should
					consider fixing these first.</li>
				<li>Values in red exceed threshold.</li>
			</xsl:with-param>
			<xsl:with-param name="tips">
				<li>The complete list is available in the Measurement perspective -
					make sure you are at the root of the
					"Drill-down" in the top-left, and do not select any items in the pie.</li>
				<li>You can integrate the generation of this summary into your build
					process. See help for details.</li>
			</xsl:with-param>
		</xsl:call-template>

	</xsl:template>

	<xsl:template match="metric[@name='Tangled']">
		<xsl:choose>
			<xsl:when test="@value &lt; @threshold">
				<xsl:value-of select="format-number(@value, $NF_PERCENTAGE)" />
			</xsl:when>
			<xsl:otherwise>
				<font color="red">
					<xsl:value-of select="format-number(@value, $NF_PERCENTAGE)" />
				</font>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="metric[@name='Fat']">
		<xsl:choose>
			<xsl:when test="@value &lt; @threshold">
				<xsl:value-of select="format-number(@value, $NF_INTEGER)" />
			</xsl:when>
			<xsl:otherwise>
				<font color="red">
					<xsl:value-of select="format-number(@value, $NF_INTEGER)" />
				</font>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="my-table">
		<xsl:param name="content" />
		<xsl:param name="width" />
		<xsl:param name="bgcolor">
			white
		</xsl:param>
		<xsl:param name="border">
			1
		</xsl:param>
		<xsl:choose>
			<xsl:when test="$INLINE='true'">
				<table style="border-style:solid;border-width:{$border}px;"
					width="{$width}" bgcolor="{$bgcolor}" cellpadding="0">
					<tr>
						<td>
							<table border="0" cellpadding="0">
								<xsl:copy-of select="$content" />
							</table>
						</td>
					</tr>
				</table>
			</xsl:when>
			<xsl:otherwise>
				<table class="inbrowser" width="{$width}">
					<xsl:copy-of select="$content" />
				</table>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="my-th">
		<xsl:param name="content" />
		<xsl:param name="align">
			right
		</xsl:param>
		<xsl:variable name="style">
			<xsl:if test="$align='right'">
				padding-left: 5
			</xsl:if>
		</xsl:variable>
		<xsl:choose>
			<xsl:when test="$INLINE='true'">
				<td align="{$align}" style="{$style}">
					<b>
						<xsl:copy-of select="$content" />
					</b>
				</td>
			</xsl:when>
			<xsl:otherwise>
				<th align="{$align}">
					<xsl:copy-of select="$content" />
				</th>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="tips">
		<xsl:param name="notes" />
		<xsl:param name="tips" />
		<xsl:if test="$TIPS='true'">
			<xsl:if test="$notes or $tips">
				<br />
				<xsl:call-template name="my-table">
					<xsl:with-param name="width">
						800
					</xsl:with-param>
					<xsl:with-param name="border">
						1
					</xsl:with-param>
					<xsl:with-param name="bgcolor">
						#FFFFE8
					</xsl:with-param>
					<xsl:with-param name="content">
						<tr bgcolor="#FFFFE8">
							<td>
								<xsl:value-of select="$SPACE" />
							</td>
							<td>
								<xsl:value-of select="$SPACE" />
							</td>
						</tr>
						<xsl:call-template name="tip">
							<xsl:with-param name="type" select="'Note'" />
							<xsl:with-param name="content" select="$notes" />
						</xsl:call-template>
						<xsl:call-template name="tip">
							<xsl:with-param name="type" select="'Tip'" />
							<xsl:with-param name="content" select="$tips" />
						</xsl:call-template>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:if>
		</xsl:if>
	</xsl:template>

	<xsl:template name="tip">
		<xsl:param name="type" />
		<xsl:param name="content" />
		<xsl:if test="$content">
			<tr bgcolor="#FFFFE8">
				<td valign="top" nowrap="nowrap">
					<small>
						<b>
							<xsl:copy-of select="$type" />
							:
						</b>
					</small>
				</td>
				<td valign="top">
					<small>
						<ul>
							<xsl:copy-of select="$content" />
						</ul>
					</small>
				</td>
			</tr>
		</xsl:if>
	</xsl:template>

</xsl:stylesheet>
<!-- Stylus Studio meta-information - (c) 2004-2005. Progress Software Corporation. 
	All rights reserved. <metaInformation> <scenarios/><MapperMetaTag><MapperInfo 
	srcSchemaPathIsRelative="yes" srcSchemaInterpretAsXML="no" destSchemaPath="" 
	destSchemaRoot="" destSchemaPathIsRelative="yes" destSchemaInterpretAsXML="no"/><MapperBlockPosition></MapperBlockPosition><TemplateContext></TemplateContext><MapperFilter 
	side="source"></MapperFilter></MapperMetaTag> </metaInformation> -->
